
#include <stdio.h>
#include <assert.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "dvh.h"

#define __u16	unsigned short
#define __u32	unsigned int

#define swab16(x) \
({ \
	__u16 __x = (x); \
	((__u16)( \
		(((__u16)(__x) & (__u16)0x00ffU) << 8) | \
		(((__u16)(__x) & (__u16)0xff00U) >> 8) )); \
})

#define swab32(x) \
({ \
	__u32 __x = (x); \
	((__u32)( \
		(((__u32)(__x) & (__u32)0x000000ffUL) << 24) | \
		(((__u32)(__x) & (__u32)0x0000ff00UL) <<  8) | \
		(((__u32)(__x) & (__u32)0x00ff0000UL) >>  8) | \
		(((__u32)(__x) & (__u32)0xff000000UL) >> 24) )); \
})

#if __BYTE_ORDER == __LITTLE_ENDIAN
#define be32_to_cpu(x) swab32((x))
#define be16_to_cpu(x) swab16((x))
#else
#define be32_to_cpu(x) ((__u32)(x))
#define be16_to_cpu(x) ((__u16)(x))
#endif

__u32 vh_calc_checksum(struct volume_header *vh) {
	__u32	oldchecksum,
		newsum=0,
		*buffer=(__u32 *) vh;
	int	i;

	oldchecksum=vh->vh_csum;
	vh->vh_csum=0;

	for(i=0;i<sizeof(struct volume_header)/sizeof(__u32);i++) {
		newsum=newsum-be32_to_cpu(buffer[i]);
	}

	vh->vh_csum=oldchecksum;
	return(newsum);
}

int main(int argc, char **argv) {
	int 			fd,
				i;
	struct volume_header	vh;

	if (argc < 1)
		exit(1);

	assert((fd=open(argv[1], O_RDONLY)));


	assert((sizeof(struct volume_header) == read(fd, &vh, sizeof(struct volume_header))));

	close(fd);


	printf("vh_magic: %08x (%s)\n", 
			be32_to_cpu(vh.vh_magic), 
			(be32_to_cpu(vh.vh_magic) == VHMAGIC) ? "correct" : "incorrect");
	printf("vh_rootpt: %04x\n",
			be16_to_cpu(vh.vh_rootpt));
	printf("vh_swappt: %04x\n",
			be16_to_cpu(vh.vh_swappt));
	printf("vh_bootfile: \"%s\"\n", 
			&vh.vh_bootfile);

	printf("device_parameters:\n");

	printf("\tdp_skew: %02x\n", vh.vh_dp.dp_skew);
	printf("\tdp_gap1: %02x\n", vh.vh_dp.dp_gap1);
	printf("\tdp_gap2: %02x\n", vh.vh_dp.dp_gap2);
	printf("\tdp_spares_cyl: %02x\n", vh.vh_dp.dp_spares_cyl);
	printf("\tdp_cyls: %04x\n", be16_to_cpu(vh.vh_dp.dp_cyls));
	printf("\tdp_shd0: %04x\n", be16_to_cpu(vh.vh_dp.dp_shd0));
	printf("\tdp_trks0: %04x\n", be16_to_cpu(vh.vh_dp.dp_trks0));
	printf("\tdp_ctq_depth: %04x\n", be16_to_cpu(vh.vh_dp.dp_ctq_depth));
	printf("\tdp_cylshi: %04x\n", be16_to_cpu(vh.vh_dp.dp_cylshi));
	printf("\tdp_unused: %04x\n", be16_to_cpu(vh.vh_dp.dp_unused));
	printf("\tdp_secs: %04x\n", be16_to_cpu(vh.vh_dp.dp_secs));
	printf("\tdp_secbytes: %04x\n", be16_to_cpu(vh.vh_dp.dp_secbytes));
	printf("\tdp_unterleave: %04x\n", be16_to_cpu(vh.vh_dp.dp_interleave));
	printf("\tdp_bflags: %08x\n", be32_to_cpu(vh.vh_dp.dp_flags));
	printf("\tdp_datarate: %08x\n", be32_to_cpu(vh.vh_dp.dp_datarate));
	printf("\tdp_nretries: %08x\n", be32_to_cpu(vh.vh_dp.dp_nretries));
	printf("\tdp_mspw: %08x\n", be32_to_cpu(vh.vh_dp.dp_mspw));
	printf("\tdp_xgap1: %04x\n", be16_to_cpu(vh.vh_dp.dp_xgap1));
	printf("\tdp_xsync: %04x\n", be16_to_cpu(vh.vh_dp.dp_xsync));
	printf("\tdp_xrdly: %04x\n", be16_to_cpu(vh.vh_dp.dp_xrdly));
	printf("\tdp_xgap2: %04x\n", be16_to_cpu(vh.vh_dp.dp_xgap2));
	printf("\tdp_xrgate: %04x\n", be16_to_cpu(vh.vh_dp.dp_xrgate));
	printf("\tdp_xwcont: %04x\n", be16_to_cpu(vh.vh_dp.dp_xwcont));

	printf("volume_directory:\n");
	for(i=0;i<NVDIR;i++) {
		printf("\tEntry %d\n", i);
		printf("\t\tvd_name:\t\"%s\"\n", vh.vh_vd[i].vd_name);
		printf("\t\tvd_lbn:\t\t%08x\n", be32_to_cpu(vh.vh_vd[i].vd_lbn));
		printf("\t\tvd_nbytes:\t%08x\n", be32_to_cpu(vh.vh_vd[i].vd_nbytes));
	}

	printf("partition_table:\n");
	for(i=0;i<NPARTAB;i++) {
		printf("\tEntry %d\n", i);
		printf("\t\tpt_nblks:\t%08x\n", be32_to_cpu(vh.vh_pt[i].pt_nblks));
		printf("\t\tpt_firstlbn:\t%08x\n", be32_to_cpu(vh.vh_pt[i].pt_firstlbn));
		printf("\t\tpt_type:\t%08x\n", be32_to_cpu(vh.vh_pt[i].pt_type));
	}

	printf("vh_csum: %08x (%s)\n", 
			be32_to_cpu(vh.vh_csum),
			(be32_to_cpu(vh.vh_csum) == vh_calc_checksum(&vh)) ? "correct" : "incorrect");
	printf("vh_fill: %08x\n", 
			be32_to_cpu(vh.vh_fill));
}
